#!/usr/bin/perl
use strict;
use PP::PP_Engine;
use PP::PP_Weighting;
use Getopt::Std;

my %argopts;
if (!getopts('i:w:o:l', \%argopts)) { printusage(); }

#get params

# Interactions
if (!defined($argopts{i})) { printusage(); }
my $int_file = $argopts{i};

# FS-Weight Threshold
if (!defined($argopts{w})) { $argopts{w} = 0.2; }
my $fsweight_threshold = $argopts{w};

# Output file
if (!defined($argopts{o})) { $argopts{o} = "output.txt"; }
my $outputfile = $argopts{o};

# Initialize prediction engine and read interaction files
my $p_engine = PP_Engine->new();

my @int_files = split(/,/, $int_file);
foreach (@int_files)
{
 $_ =~ s/^\s+//;
 $_ =~ s/\s+$//;
 print "# Interactions: $_\n";
 $p_engine->ReadDataSource($_, 'Protein Interaction');
}

print "# FS-Weight Threshold: $fsweight_threshold\n";

# Compute L2 interaction weights
my $added = Expand($p_engine, defined($argopts{l})?1:0);
print "$added\n";

$p_engine->PrintSummary();

my $key;
open DAT, ">$outputfile";
foreach $key(keys %{$p_engine->{INTERACTIONS}})
{
 my ($i, $j) = split(/\|/, $key);
 print DAT "$i\t$j\t".$p_engine->{ENTITIES}{$i}{NEIGHBOURS}{$j}."\n"; 
}
close DAT;

###############################################################
sub Expand
{
 my ($p_engine, $level2) = @_;
 # Compute FS-Weight
 my $weight_manager = PP_Weighting->new($p_engine);
 my %completed = ();
 my $id;
 my $neighbour;
 foreach $id (keys %{$p_engine->{ENTITIES}})
 {
  my %neighbours;
  foreach $neighbour(keys %{$p_engine->{ENTITIES}{$id}{NEIGHBOURS}})
  {
   $neighbours{$neighbour} = 1;
   if ($level2)
   {
    my $l2neighbour;
    foreach $l2neighbour(keys %{$p_engine->{ENTITIES}{$neighbour}{NEIGHBOURS}})
    {
     if ($l2neighbour ne $id) { 
      $neighbours{$l2neighbour} = 1; 
     }
    }
   }
  }

  foreach $neighbour(keys %neighbours)
  {
   if (defined($completed{$neighbour})) { next; }
   if (defined($p_engine->{ENTITIES}{$id}{NEIGHBOURS}{$neighbour}))
   {
    $p_engine->{ENTITIES}{$id}{NEIGHBOURS}{$neighbour} =
    $p_engine->{ENTITIES}{$neighbour}{NEIGHBOURS}{$id} 
    = $weight_manager->ComputeFSWeight($id, $neighbour, 1);
   }
   else
   {
    $p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'}{$neighbour} =
    $p_engine->{ENTITIES}{$neighbour}{'L2NEIGHBOURS'}{$id} 
    = $weight_manager->ComputeFSWeight($id, $neighbour, 1);
   }
  }
  $completed{$id} = 1;
 }
 
 my $added = 0;

 foreach $id (keys %{$p_engine->{ENTITIES}})
 {
  foreach $neighbour(keys %{$p_engine->{ENTITIES}{$id}{NEIGHBOURS}})
  {
   my $key = $p_engine->GetInteractionKey($id, $neighbour);  
   if ($p_engine->{ENTITIES}{$id}{NEIGHBOURS}{$neighbour} < $fsweight_threshold)
   {   
    delete $p_engine->{ENTITIES}{$id}{NEIGHBOURS}{$neighbour};
    delete $p_engine->{INTERACTIONS}{$key};
   }
  }

  foreach $neighbour(keys %{$p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'}})
  {
   if ($p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'}{$neighbour} >= $fsweight_threshold)
   {
    $p_engine->{ENTITIES}{$id}{NEIGHBOURS}{$neighbour} = $p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'}{$neighbour};
    my $key = $p_engine->GetInteractionKey($id, $neighbour);
    push @{$p_engine->{INTERACTIONS}{$key}{DATASOURCE}{'Protein Interaction'}{'L2NEIGHBOURS'}{'scores'}}, $p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'}{$neighbour};
    $added ++;
   }
   delete $p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'}{$neighbour}
  }
  delete $p_engine->{ENTITIES}{$id}{'L2NEIGHBOURS'};
 }
 return $added;
}

sub printusage
{   
 print "\n";
 print "usage: filter_int -i interactions [-w fsweight_threshold] [-l] [-o output_file]\n\n";
 print "options:\n\n";
 print "-i :\tInteraction file\n";
 print "-l :\tInclude Indirect Interactions\n";
 print "-w :\tFS-Weight threshold for filtering interactions (default = 0.2)\n";
 print "-o :\tOutput interaction file (default = output.txt)\n";
 print "\n";
 exit;
}

